/*********************************************************************

          ps.c: PolhemusSensor device driver
                    for PATRIOT W/USB (made from FASTRAK ver.)
                                        Kenji with Tail

*********************************************************************/


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <termios.h>
#include "ps.h"


#define _DEBUG_ 1


#define CTL_COM(n) ((n) - 0x40)
static	int	rdPort, wrPort;
struct	termios	initialAtt;


/********************************************************************/


/* ChangeUnit_ps() changes unit inchi/meter
 *    Returns False (zero) if buffer is full
 *    Returns True (non-zero) if successful
 *    give unit:  inchi: INCHI_PS/ meter: METER_PS.
 */
#define INCHI_PS 0
#define METER_PS 1
int ChangeUnit_ps(int unit);

/* ChangeFormat_ps() changes format ascii/binary
 *    Returns False (zero) if buffer is full
 *    Returns True (non-zero) if successful
 *    give form:  ascii: ASCII_PS/ binary: BINARY_PS.
 */
#define ASCII_PS 0
#define BINARY_PS 1
int ChangeFormat_ps(int form);


int Init_ps(char *port, long int baud)
{
	struct termios newAtt;

	rdPort=open(port,O_RDONLY|O_NDELAY);
	wrPort=open(port,O_WRONLY);
	if ((rdPort==-1) || (wrPort==-1)){
		printf("Init_ps: Error connecting to Patriot\n");
		return 0;
	}

	// set up terminal for raw data
	tcgetattr(rdPort,&initialAtt);	// save this to restore later
	newAtt=initialAtt;
	cfmakeraw(&newAtt);
	if (tcsetattr(rdPort,TCSANOW,&newAtt)){
		printf("Init_ps: Error setting terminal attributes\n");
		close(rdPort);
		close(wrPort);
		return 0;
	}

	printf("Init_ps: Connected to Patriot\n");

	ChangeFormat_ps(ASCII_PS);
	ChangeUnit_ps(METER_PS);
	SetStation_ps(1, ON_PS);
	SetStation_ps(2, ON_PS);

	return 1;
}


int Close_ps(void)
{
	tcsetattr(rdPort,TCSANOW,&initialAtt);	// restore the original attributes
	close(rdPort);
	close(wrPort);
	return 1;
}


int Reset_ps(void)
{
	return 1;
}


int ChangeUnit_ps(int unit)
{
	switch(unit){
	    case INCHI_PS:
		return (write(wrPort, "u0\r", 3) == 3);
	    case METER_PS:
		return (write(wrPort, "u1\r", 3) == 3);
	    default:
		fprintf(stderr, "ChangeUnit_ps: please give INCHI_PS/METER_PS.\n");
		return -1;
	}
}


int ChangeFormat_ps(int form)
{
	switch(form){
	    case ASCII_PS:
		return (write(wrPort, "f0\r", 3) == 3);
	    case BINARY_PS:
		//return (write(wrPort, "f1\r", 3) == 3);
	        fprintf(stderr, "ChangeFormat_ps: BINARY mode is not supported.\n");
		return -1;
	    default:
		fprintf(stderr, "ChangeFormat_ps: please give ASCII_PS/BINARY_PS.\n");
		return -1;
	}
}


int SetStation_ps(int stn, int act)
{
	int result;
	char stystr[] = "o1,2,4,10\r";
	char frmstr[] = "o2,2,4\r";

	if(stn == 1){
		return (write(wrPort, stystr, 10) == 10); /* set stylus data */
	} else {
		return (write(wrPort, frmstr, 7) == 7); /* set normal data */
	}

	return 1;
}


/* Returns 1+2 if stl#1 + stl#2 are ON (*) */
int PosRequest_ps(PosOri_ps p1, PosOri_ps p2, PosOri_ps p3, PosOri_ps p4)
{
	char rcv[3], btn='0';
	int br,count;
	// may need to be larger if getting alot of data
	static char buf[1000];

	read(rdPort,buf,1000);
	memset(buf,0,1000);
	
	count=0;
	write(wrPort,"p",1);			// request data
	do {
		br=read(rdPort,buf,1000);
		usleep(1000);			// wait 1ms
	} while ((br<=0) && (count++<500));	// try to read 500 times

	buf[br]='\0';	//terminate

	if (br > 110){ // 119
		sscanf(buf, "%s %f %f %f %f %f %f %c %s %f %f %f %f %f %f",
		       rcv, p1, p1+1, p1+2, p1+3, p1+4, p1+5, &btn,
		       rcv, p2, p2+1, p2+2, p2+3, p2+4, p2+5);
	} else if (br > 100){ // 116
		sscanf(buf, "%s %f %f %f %f %f %f %s %f %f %f %f %f %f",
		       rcv, p1, p1+1, p1+2, p1+3, p1+4, p1+5,
		       rcv, p2, p2+1, p2+2, p2+3, p2+4, p2+5);
	} else if (br > 60){ // 61
		if (buf[1] == '1'){
			sscanf(buf, "%s %f %f %f %f %f %f %c",
			       rcv, p1, p1+1, p1+2, p1+3, p1+4, p1+5, &btn);
		} else if (buf[1] == '2'){
			sscanf(buf, "%s %f %f %f %f %f %f %c",
			       rcv, p2, p2+1, p2+2, p2+3, p2+4, p2+5, &btn);
			btn++;
		}
	} else if (br > 50){ // 58
		if (buf[1] == '1'){
			sscanf(buf, "%s %f %f %f %f %f %f",
			       rcv, p1, p1+1, p1+2, p1+3, p1+4, p1+5);
		} else if (buf[1] == '2'){
			sscanf(buf, "%s %f %f %f %f %f %f",
			       rcv, p2, p2+1, p2+2, p2+3, p2+4, p2+5);
		}
	} else {
		return -1;
	}

	return (int) btn - '0';
}

